<?php

namespace Shahid\Bin;

use FFMpeg\FFMpeg;
use FFMpeg\Media\AdvancedMedia;
use FFMpegMy\Media\VideoMy;
use FFMpegMy\Media\X264My;
use Shahid\Command\console;

/**
 * Class FFMpegBin
 * @package Shahid\Bin
 * @author MHK
 */
class FFMpegBin
{
    protected static $FFMpeg;
    protected static $path;

    /**
     * @param $path
     */
    public static function setBinPath($path)
    {
        self::$path = $path;
    }

    /**
     * @param $uri
     * @param $file_name
     * @param null $audio_uri
     */
    public static function run($uri, $file_name, $audio_uri = null)
    {
        self::makeMedia($uri, $file_name, $audio_uri)->save();
    }

    /**
     * @param $uri
     * @param $file_name
     * @param null $audio_uri
     *
     * @return AdvancedMedia
     */
    protected static function makeMedia($uri, $file_name, $audio_uri = null)
    {
        $AdvancedMedia = new AdvancedMedia(array_merge([$uri], (empty($audio_uri) ? [] : [$audio_uri])), self::getFFMpeg()->getFFMpegDriver(), self::getFFMpeg()->getFFProbe());
        $AdvancedMedia->setAdditionalParameters([
            '-c', 'copy',
            '-acodec', 'copy',
            '-stats',
            $file_name,
        ]);
        return $AdvancedMedia;
    }

    /**
     * @return FFMpeg
     */
    public static function getFFMpeg()
    {
        if (is_null(self::$FFMpeg))
            self::$FFMpeg = FFMpeg::create([
                'ffmpeg.binaries' => self::getPath() . 'ffmpeg.exe', // the path to the FFMpeg binary
                'ffprobe.binaries' => self::getPath() . 'ffprobe.exe', // the path to the FFProbe binary
                'timeout' => 3600, // the timeout for the underlying process
                //'ffmpeg.threads' => 12,   // the number of threads that FFMpeg should use
            ]);
        return self::$FFMpeg;
    }

    /**
     * @return string
     */
    protected static function getPath()
    {
        return !empty(self::$path) ? self::$path . DIRECTORY_SEPARATOR : '/bin/';
    }

    /**
     * @param $uri
     * @param $file_name
     * @param null $audio_uri
     *
     * @return string
     */
    public static function command($uri, $file_name, $audio_uri = null)
    {
        return 'ffmpeg' . self::makeMedia($uri, $file_name, $audio_uri)->getFinalCommand();
    }

    /**
     * @param $uri
     * @param $fileName
     * @param null $audio_uri
     */
    public static function runOld($uri, $fileName, $audio_uri = null)
    {
        console::resetCHCP();
        $my_video = new VideoMy($uri, self::getFFMpeg()->getFFMpegDriver(), self::getFFMpeg()->getFFProbe());
        $format = new X264My('copy', 'copy');
        $format->setAdditionalParameters((empty($audio_uri) ? [
            '-c', 'copy',
            //'-bsf:a',
            //'aac_adtstoasc',
            '-acodec', 'copy',
        ] : [
            '-i', $audio_uri,
            '-c', 'copy',
            '-acodec', 'copy',
        ]));
        $format->on('progress', function ($video, $format, $percentage) use ($fileName) {
            console::progressBar($percentage);
        });
        $my_video->save($format, $fileName);
    }
}