<?php
/**
 * User: MHK
 * Date: 20/05/2021
 */

namespace Shahid\Video;

use Shahid\Bin\FFMpegBin;
use Symfony\Component\Console\Helper\ProgressBar;
use Symfony\Component\Console\Output\ConsoleOutput;
use Symfony\Component\Process\Exception\ProcessFailedException;
use Symfony\Component\Process\ExecutableFinder;
use Symfony\Component\Process\Process;

class FFMpegCli
{
    /**
     * @var array
     */
    protected static $paths = [];

    /**
     * @param $path
     */
    public static function addPath($path)
    {
        self::$paths[] = $path;
    }

    /**
     * @param $uri
     * @param $file_name
     * @param null $audio_uri
     */
    public static function run($uri, $file_name, $audio_uri = null)
    {
        self::exec(array_merge_recursive([
            '-y',
            '-i', $uri,
        ], (empty($audio_uri) ? [] : [
            '-i', $audio_uri,
        ]), [
            '-c', 'copy',
            '-err_detect', 'explode',
            $file_name
        ]));
    }

    /**
     * @param array $opts
     */
    protected static function exec(array $opts)
    {
        // creates a new progress bar (50 units)
        $progressBar = new ProgressBar(new ConsoleOutput);
        $progressBar->setFormat('debug');
        $process = new Process(array_merge([self::findFFMpeg()], $opts));
        $process->setTimeout(0);
        $process->start();
        $progressBar->start();
        $ffout = '';
        $process->wait(function ($type, $buffer) use ($progressBar, &$ffout) {
            $regex = "/frame= *(\d+).*size=.*(\d+\w+).*time=([\w:\.]+).*bitrate= *([\w:\.]+).*speed=([\w:\.]+)/i";
            preg_match($regex, $buffer, $matches);
            if (isset($matches[0])) $ffout = "frame= {$matches[1]}    time= {$matches[3]}    bitrate= {$matches[4]}    speed= {$matches[5]}";
            $progressBar->setFormat(' %elapsed:6s% [%bar%]' . (empty($ffout) ? '' : " {$ffout}"));
            $progressBar->advance();
        });
        $progressBar->finish();
        // executes after the command finishes
        if (!$process->isSuccessful()) throw new ProcessFailedException($process);
    }

    /**
     * @return string|null
     */
    public static function findFFMpeg()
    {
        $executableFinder = new ExecutableFinder();
        return $executableFinder->find('ffmpeg', FFMpegBin::getFFMpeg()->getFFMpegDriver()->getProcessBuilderFactory()->getBinary(), self::getPaths());
    }

    /**
     * @return array
     */
    public static function getPaths()
    {
        return self::$paths;
    }

    /**
     * @param $uri
     * @param $file_name
     * @param null $audio_uri
     *
     * @return array|string
     */
    public static function getCommandLine($uri, $file_name, $audio_uri = null)
    {
        $process = new Process(array_merge_recursive([
            self::findFFMpeg(),
            '-y',
            '-i', $uri,
        ], (empty($audio_uri) ? [] : [
            '-i', $audio_uri,
        ]), [
            '-c', 'copy',
            '-err_detect', 'explode',
            //'-acodec', 'copy',
            $file_name
        ]));
        return $process->getCommandLine();
    }

    /**
     * @param $uri
     * @param $file_name
     */
    public static function subtitle($uri, $file_name)
    {
        self::exec([
            '-y',
            '-i', $uri,
            '-scodec', 'srt',
            $file_name
        ]);
    }
}