<?php

namespace Shahid\Video;

use Chrisyue\PhpM3u8\Facade\ParserFacade;
use Chrisyue\PhpM3u8\Stream\TextStream;
use Shahid\Command\console;
use Shahid\Command\UserChose;
use Symfony\Component\DomCrawler\Crawler;

/**
 * Class Manifest
 * @package Shahid\Video
 * @author MHK
 */
class Manifest
{
    /**
     * @var string
     */
    protected $url;
    /**
     * @var string
     */
    protected $content;
    /**
     * @var \Illuminate\Support\Collection|\Shahid\Video\Quality[]
     */
    protected $qualities;
    /**
     * @var \Illuminate\Support\Collection|\Shahid\Video\Audio[]
     */
    protected $audios;
    /**
     * @var \Illuminate\Support\Collection|\Shahid\Video\Subtitle[]
     */
    protected $subtitles;
    /**
     * @var ParserFacade
     */
    protected $parser;
    protected $successful = false;

    /**
     * Manifest constructor.
     *
     * @param string $url
     * @param null $name
     *
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function __construct(string $url, $name = null)
    {
        $this->qualities = collect();
        $this->audios = collect();
        $this->subtitles = collect();
        $this->parser = new ParserFacade();
        $this->url = $url;
        $response = WebClient::client()->request('GET', $this->url);
        $this->content = $response->getBody()->getContents();
        /**
         * XML checking (.ism)
         */
        if (@simplexml_load_string($this->content)) {
            console::text("<error>Video format is not supported !</error>" . (empty($name) ? null : " : {$name}"));
            return;
        } elseif (0/*ism*/) {
            return;
            $crawler = new Crawler($this->content);
            foreach ($crawler->children('StreamIndex[Type="video"]')->first()->children('QualityLevel') as $QualityLevel) {
                $QualityLevel->attr('Bitrate');
                $QualityLevel->attr('MaxHeight');
                $qualities[] = $Quality = new Quality($QualityLevel['RESOLUTION'], $QualityLevel['BANDWIDTH'], "{$this->dirname()}{$QualityLevel['uri']}");
            }
        } else {
            /**
             * @var \ArrayObject $mediaPlaylist
             */
            $mediaPlaylist = $this->parser->parse(new TextStream($this->content));
            foreach ($mediaPlaylist->offsetGet('EXT-X-STREAM-INF') as $QualityLevel) {
                if (empty($QualityLevel['RESOLUTION'])) continue;
                $this->qualities->push(new Quality(
                        $QualityLevel['RESOLUTION'],
                        $QualityLevel['BANDWIDTH'],
                        "{$this->dirname()}{$QualityLevel['uri']}",
                        $QualityLevel['AVERAGE-BANDWIDTH'] ?? null,
                        $QualityLevel['FRAME-RATE'] ?? null,
                        $QualityLevel['CODECS'] ?? null
                    )
                );
            }
            $this->qualities = $this->qualities->sortByDesc(function (Quality $quality) {
                return $quality->index();
            })->values();
            if ($mediaPlaylist->offsetExists('EXT-X-MEDIA')) foreach ($mediaPlaylist->offsetGet('EXT-X-MEDIA') as $media) {
                switch ($media['TYPE']) {
                    case 'AUDIO':
                        $this->audios->push(new Audio(
                                "{$this->dirname()}{$media['URI']}",
                                $media['GROUP-ID'],
                                $media['CHANNELS'],
                                $media['NAME'],
                                $media['LANGUAGE'],
                                $media['DEFAULT'] ?? 'NO',
                                $media['AUTOSELECT'] ?? 'NO'
                            )
                        );
                        break;
                    case 'SUBTITLES':
                        $this->subtitles->push(new Subtitle(
                                "{$this->dirname()}{$media['URI']}",
                                $media['GROUP-ID'],
                                $media['NAME'],
                                $media['LANGUAGE'],
                                $media['DEFAULT'] ?? 'NO',
                                $media['AUTOSELECT'] ?? 'NO',
                                $media['FORCED'] ?? 'NO'
                            )
                        );
                        break;
                }
            }
            $this->successful = true;
        }
    }

    /**
     * @return string|null
     */
    protected function dirname()
    {
        $dirname = pathinfo($this->url)['dirname'] ?? null;
        return $dirname ? "{$dirname}/" : null;
    }

    /**
     * @return \Illuminate\Support\Collection|Audio[]
     */
    public function getAudios()
    {
        return $this->audios;
    }

    /**
     * @return \Illuminate\Support\Collection|Subtitle[]
     */
    public function getSubtitles()
    {
        return $this->subtitles;
    }

    /**
     * @return bool
     */
    public function isSuccessful(): bool
    {
        return $this->successful;
    }

    /**
     * @return string
     */
    public function getUrl()
    {
        return $this->url;
    }

    /**
     * @return string
     */
    public function getContent()
    {
        return $this->content;
    }

    /**
     * @return \Illuminate\Support\Collection|\Shahid\Video\Quality[]
     */
    public function getQualities()
    {
        return $this->qualities;
    }

    /**
     * @param string|null $quality 'best' 'worst' '1080' '1080p' '720p' '468' ...
     *
     * @return mixed|Quality|null
     * @throws \Exception
     */
    public function getQuality(string $quality = null)
    {
        switch ($quality) {
            case 'best':
                return $this->qualities->first();
            case 'worst':
                return $this->qualities->last();
            default:
                $quality = str_replace('p', null, $quality);
                if (!empty($quality)) {
                    if ($qlty = $this->getQualityByHeight($quality))
                        return $qlty;
                    else goto ask;
                } else {
                    ask:
                    $choises = [];
                    $this->qualities->each(function (Quality $quality) use (&$choises) {
                        $choises[] = "{$quality}";
                    });
                    $selected = UserChose::chose($choises);
                    return $this->qualities->get($selected);
                }
        }
    }

    /**
     * @param $height
     *
     * @return Quality|null
     */
    protected function getQualityByHeight($height)
    {
        return $this->qualities->filter(function (Quality $quality) use ($height) {
            return $quality->getResolution()->getHeight() == $height;
        })->first();
    }

    /**
     * @param null $language
     * @param bool $no_audio_choise
     *
     * @return false|Audio
     * @throws \Exception
     */
    public function getAudio($language = null, bool $no_audio_choise = true)
    {
        if ($language === false || !$this->audios->count()) return false;
        if (($exists = $this->audios->filter(function (Audio $audio) use (&$language) {
            return $audio->getNAME() == $language;
        }))->count()) return $exists->first();
        switch ($language) {
            case 'default':
                $default = $this->audios->filter(function (Audio $audio) {
                    return $audio->isDEFAULT();
                })->first();
                if (!$default) goto ask;
                return $default;
            default:
                ask:
                $choises = [];
                $this->audios->each(function (Audio $audio) use (&$choises) {
                    $choises[] = $audio->getNAME();
                });
                if ($no_audio_choise) $choises[] = 'No audio';
                $nop = array_key_last($choises);
                $selected = UserChose::chose($choises);
                if ($nop == $selected) return false;
                return $this->audios->get($selected);
        }
    }

    /**
     * @param null $language
     * @param bool $no_subtitle_choise
     *
     * @return false|Subtitle
     * @throws \Exception
     */
    public function getSubtitle($language = null, bool $no_subtitle_choise = true)
    {
        if ($language === false || !$this->subtitles->count()) return false;
        if (($exists = $this->subtitles->filter(function (Subtitle $subtitle) use (&$language) {
            return $subtitle->getNAME() == $language;
        }))->count()) return $exists->first();
        switch ($language) {
            case 'default':
                $default = $this->subtitles->filter(function (Subtitle $subtitle) {
                    return $subtitle->isDEFAULT();
                })->first();
                if (!$default) goto ask;
                return $default;
            default:
                ask:
                $choises = [];
                $this->subtitles->each(function (Subtitle $subtitle) use (&$choises) {
                    $choises[] = $subtitle->getNAME();
                });
                if ($no_subtitle_choise) {
                    $choises[] = 'No subtitle';
                    $nop = array_key_last($choises);
                }
                $selected = UserChose::chose($choises);
                if (isset($nop) && $nop == $selected) return false;
                return $this->subtitles->get($selected);
        }
    }
}