<?php

namespace Shahid\Api;


use Shahid\Command\console;
use Shahid\Video\WebClient;

/**
 * Class PlayLoad
 * @package Shahid\Api
 * @author MHK
 */
class PlayLoad
{
    const API_URL = 'https://api2.shahid.net/proxy/v2/playout/new/url/{id}';
    protected $id;
    protected $playout;
    protected $successful = false;

    /**
     * PlayLoad constructor.
     *
     * @param $id
     */
    public function __construct($id)
    {
        $this->id = $id;
    }

    /**
     * @return bool
     */
    public function isSuccessful(): bool
    {
        return $this->successful;
    }

    /**
     * @return string|string[]|null
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function getM3u8()
    {
        $m3u8 = $this->data()->playout->url ?? null;
        return str_replace(['_720', '-720'], ['_1080', '-1080'], $m3u8);
    }

    /**
     * @return mixed|null
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public function data()
    {
        if (!empty($this->playout)) return $this->playout;
        else {
            try {
                $response = WebClient::client()->request('GET', $this->apiUrl());
                if ($response->getStatusCode() == 200) {
                    $contents = $response->getBody()->getContents();
                    if (empty($contents)) return null;
                    $this->successful = true;
                    return $this->playout = json_decode($contents, 0);
                } else
                    throw new  \Exception($response->getBody(), $response->getStatusCode());
            } catch (\GuzzleHttp\Exception\ClientException $e) {
                $response = $e->getResponse();
                $responseBodyAsString = $response->getBody()->getContents();
                if (empty($responseBodyAsString)) return null;
                $json = json_decode($responseBodyAsString);
                switch ($response->getStatusCode()) {
                    case 422:
                        if (console::isCli()) console::text("<error>{$json->faults[0]->internalMessage}</error> : <info>{$json->faults[0]->userMessage}</info>");
                        else dd([$json->faults[0]->internalMessage => $json->faults[0]->userMessage]);
                        return null;
                }
                return null;
            } catch (\Exception $exception) {
                if (console::isCli()) console::text("<error>{$exception->getCode()}: {$exception->getMessage()}</error><info>");
                else dd([$exception->getCode() => $exception->getMessage()]);
                return null;
            }
        }
    }

    /**
     * @return mixed|null
     */
    protected function apiUrl()
    {
        if (!$this->getId()) return null;
        return str_replace('{id}', $this->getId(), self::API_URL);
    }

    /**
     * @return mixed
     */
    public function getId()
    {
        return $this->id;
    }
}