<?php

namespace Shahid\Video;

use Chrisyue\PhpM3u8\Data\Value\Attribute\Resolution;

/**
 * Class Quality
 * @package Shahid\Video
 */
class Quality
{
    /**
     * @var \Chrisyue\PhpM3u8\Data\Value\Attribute\Resolution $RESOLUTION
     */
    protected $RESOLUTION;
    protected $BANDWIDTH;
    protected $uri;
    protected $AVERAGE_BANDWIDTH;
    protected $FRAME_RATE;
    protected $CODECS;

    /**
     * Quality constructor.
     *
     * @param $RESOLUTION
     * @param $BANDWIDTH
     * @param $uri
     * @param null $AVERAGE_BANDWIDTH
     * @param null $FRAME_RATE
     * @param null $CODECS
     */
    public function __construct(Resolution $RESOLUTION, $BANDWIDTH, $uri, $AVERAGE_BANDWIDTH = null, $FRAME_RATE = null, $CODECS = null)
    {
        $this->RESOLUTION = $RESOLUTION;
        $this->BANDWIDTH = $BANDWIDTH;
        $this->uri = $uri;
        $this->AVERAGE_BANDWIDTH = $AVERAGE_BANDWIDTH;
        $this->FRAME_RATE = $FRAME_RATE;
        $this->CODECS = $CODECS;
    }

    /**
     * @return float|int
     */
    public function index()
    {
        return $this->getResolution()->getHeight() * $this->getBandwidth();
    }

    /**
     * @return Resolution
     */
    public function getResolution(): Resolution
    {
        return $this->RESOLUTION;
    }

    /**
     * @return mixed
     */
    public function getBandwidth()
    {
        return $this->BANDWIDTH;
    }

    /**
     * @return mixed
     */
    public function getUri()
    {
        return $this->uri;
    }

    /**
     * @return null
     */
    public function getAverageBandwidth()
    {
        return $this->AVERAGE_BANDWIDTH;
    }

    /**
     * @return null
     */
    public function getFrameRate()
    {
        return $this->FRAME_RATE;
    }

    /**
     * @return null
     */
    public function getCodecs()
    {
        return $this->CODECS;
    }

    /**
     * @return string
     */
    public function __toString()
    {
        return "Quality {$this->p()} : {$this->bps()}";
    }

    /**
     * @return string
     */
    public function p()
    {
        return "{$this->getHeight()}p";
    }

    /**
     * @return int
     */
    public function getHeight()
    {
        return $this->RESOLUTION->getHeight();
    }

    /**
     * @return string
     */
    public function bps()
    {
        return "{$this->getBandwidth()}bps";
    }
}