<?php

namespace Shahid\Video;

use GuzzleHttp\Client;
use Shahid\Api\Token;

/**
 * Class Serie
 * @package Shahid\Video
 * @depricated
 */
class Serie
{
    const API_PLAYABLE_ASSET_URL = 'https://api2.shahid.net/proxy/v2.1/playableAsset';
    const API_PLAYLIST_URL = 'https://api2.shahid.net/proxy/v2.1/product/playlist';
    const URLS
        = [
            [
                'ID_URL' => '/^https?:\/\/shahid\.mbc\.net\/\w+\/series\/[\w\d%-]+\/series-(\d+)$/',
                'NAME_URL' => '/^https?:\/\/shahid\.mbc\.net\/\w+\/series\/([\w\d%-]+)\/series-\d+$/',
            ],
        ];
    public $default_quality;
    protected $index = -1;

    /**
     * Serie constructor.
     *
     * @param $url
     * @param null $default_quality
     * @depricated
     */
    public function __construct($url, $default_quality = null)
    {
        $this->url = $url;
        $this->default_quality = str_replace('p', null, $default_quality);
        $this->setIndex();
        $this->client = new Client([
            'headers' => array_merge([], Token::header()),
        ]);
    }

    /**
     * @depricated
     */
    protected function setIndex()
    {
        foreach ($this::URLS as $index => $URL) {
            preg_match($URL['ID_URL'], $this->url, $matches);
            if (count($matches)) {
                $this->index = $index;
                break;
            }
        }
    }

    /**
     * @return string|null
     * @depricated
     */
    public function getNAME()
    {
        if (!$this->getNAME_URL()) return null;
        preg_match($this->getNAME_URL(), $this->url, $matches);
        return urldecode($matches[1]) ?? null;
    }

    /**
     * @return mixed|null
     * @depricated
     */
    protected function getNAME_URL()
    {
        return $this::URLS[$this->index]['NAME_URL'] ?? null;
    }

    public function test()
    {
        $response = $this->client->request('GET', $this->apiPlayableAssetUrl(), [
            'request' => "{\"playListId\":826721000032751,\"pageNumber\":0,\"pageSize\":1000,\"sorts\":[{\"order\":\"DESC\",\"type\":\"SORTDATE\"}]}",
        ]);
        $content = $response->getBody()->getContents();
        return $content;
    }

    /**
     * @return string|string[]|null
     * @depricated
     */
    protected function apiPlayableAssetUrl()
    {
        if (!$this->getId()) return null;
        return str_replace('{id}', $this->getId(), $this::API_PLAYABLE_ASSET_URL);
    }

    /**
     * @return string|string[]|null
     * @depricated
     */
    protected function apiPlaylistUrl()
    {
        if (!$this->getId()) return null;
        return str_replace('{id}', $this->getId(), $this::API_PLAYLIST_URL);
    }

    /**
     * @return mixed|null
     * @depricated
     */
    public function getID()
    {
        if (!$this->getID_URL()) return null;
        preg_match($this->getID_URL(), $this->url, $matches);
        return $matches[1] ?? null;
    }

    /**
     * @return mixed|null
     * @depricated
     */
    protected function getID_URL()
    {
        return $this::URLS[$this->index]['ID_URL'] ?? null;
    }
}