<?php

namespace Shahid\Video;

use Shahid\Api\PlayLoad;
use Shahid\Command\console;

/**
 * Class Shahid
 * @package FFMpegMy\Media
 * @author MHK
 */
abstract class Shahid
{
    const MAX_PHP_VERSION = '7.4.0';
    const MIN_PHP_VERSION = '7.2.0';

    /**
     * @return bool
     */
    public static function isPhpSupportedVersion()
    {
        return version_compare(PHP_VERSION, self::MIN_PHP_VERSION, '>=') && version_compare(PHP_VERSION, self::MAX_PHP_VERSION, '<');
    }

    /**
     * @param int $id
     * @param $name
     * @param null $quality
     * @param null $audio
     *
     * @return array|false|string|void
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public static function getCommandLineById(int $id, $name, $quality = null, $audio = null)
    {
        if (1 == 0/*check id*/) {
            console::text("<error>ID is invalid or not supported !</error> : <info>\r\n{$id}</info>");
            return false;
        }
        return self::getCommandLineByUrl("https://shahid.mbc.net/ar/movies/{$name}/movie-player-{$id}", $name, $quality, $audio);
    }

    /**
     * @param $url
     * @param null $name
     * @param null $quality
     * @param null $audio
     *
     * @return array|false|string|void
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public static function getCommandLineByUrl($url, $name = null, $quality = null, $audio = null)
    {
        if (!$class = self::checkUrlType($url)) {
            console::text("<error>URL is invalid or not supported !</error> : <info>\r\n{$url}</info>");
            return false;
        }
        return self::getCommandLineBy(new $class($url, $name, $quality, $audio));
    }

    /**
     * @param $uri
     *
     * @return false|string
     */
    protected static function checkUrlType($uri)
    {
        foreach (Episode::URLS as $url) {
            preg_match($url['ID_URL'], $uri, $matches);
            if (count($matches)) {
                return Episode::class;
            }
        }
        foreach (Movie::URLS as $url) {
            preg_match($url['ID_URL'], $uri, $matches);
            if (count($matches)) {
                return Movie::class;
            }
        }
        return false;
    }

    /**
     * @param shahidable $shahidable
     *
     * @return array|string|void
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public static function getCommandLineBy(shahidable $shahidable)
    {
        console::text("<info>{$shahidable->getNAME()}</info>");
        console::saveCHCP();
        $PlayLoad = new PlayLoad($shahidable->getID());
        $Manifest = new Manifest($PlayLoad->getM3u8());
        if (!$Manifest->isSuccessful()) {
            console::resetCHCP();
            return;
        }
        $quality = $Manifest->getQuality($shahidable->getDefaultQuality());
        $audio = $Manifest->getAudio($shahidable->getDefaultAudio());
        console::resetCHCP();
        $fileName = self::getFileName($shahidable, $quality);
        console::text("<question>{$quality}</question>");
        if ($audio) console::text("<question>{$audio}</question>");
        return FFMpegCli::getCommandLine($quality->getUri(), $fileName, $audio ? $audio->getURI() : null);
    }

    /**
     * @param shahidable $shahidable
     * @param Quality $quality
     *
     * @return string
     */
    protected static function getFileName(shahidable $shahidable, Quality $quality)
    {
        $extra_title = "[{$quality->p()}][{$quality->bps()}]";
        return !empty($shahidable->name) ? "{$shahidable->name}{$extra_title}.mp4" : "{$shahidable->getNAME()}{$extra_title}.mp4";
    }

    /**
     * @param int $id
     * @param $name
     * @param null $quality 'best' 'worst' '1080' '1080p' '720p' '468' ...
     * @param null $audio
     *
     * @return false
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public static function downloadById(int $id, $name, $quality = null, $audio = null)
    {
        if (1 == 0/*check id*/) {
            console::text("<error>ID is invalid or not supported !</error> : <info>\r\n{$id}</info>");
            return false;
        }
        self::downloadByUrl("https://shahid.mbc.net/ar/movies/{$name}/movie-player-{$id}", $name, $quality, $audio);
    }

    /**
     * @param $url
     * @param null $name
     * @param null $quality 'best' 'worst' '1080' '1080p' '720p' '468' ...
     * @param null $audio 'ar' 'en' 'it' ...
     *
     * @return false
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public static function downloadByUrl($url, $name = null, $quality = null, $audio = null)
    {
        if (!$class = self::checkUrlType($url)) {
            console::text("<error>URL is invalid or not supported !</error> : <info>\r\n{$url}</info>");
            return false;
        }
        self::downloadBy(new $class($url, $name, $quality, $audio));
    }

    /**
     * @param shahidable $shahidable
     *
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public static function downloadBy(shahidable $shahidable)
    {
        console::text("<info>{$shahidable->getNAME()}</info>");
        console::saveCHCP();
        $PlayLoad = new PlayLoad($shahidable->getID());
        $Manifest = new Manifest($PlayLoad->getM3u8());
        if (!$Manifest->isSuccessful()) {
            console::resetCHCP();
            return;
        }
        $quality = $Manifest->getQuality($shahidable->getDefaultQuality());
        $audio = $Manifest->getAudio($shahidable->getDefaultAudio());
        console::resetCHCP();
        $fileName = self::getFileName($shahidable, $quality);
        console::text("<question>{$quality}</question>");
        $aud = null;
        if ($audio) {
            console::text("<question>{$audio}</question>");
            $aud = '+ audio';
        }
        console::text("<comment>Downloading: </comment><info>{$shahidable->getNAME()} : video {$aud}</info>");
        FFMpegCli::run($quality->getUri(), $fileName, $audio ? $audio->getURI() : null);
    }

    /**
     * @param int $id
     * @param $name
     * @param null $audio
     *
     * @return false
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public static function audioById(int $id, $name, $audio = null)
    {
        if (1 == 0/*check id*/) {
            console::text("<error>ID is invalid or not supported !</error> : <info>\r\n{$id}</info>");
            return false;
        }
        self::audioByUrl("https://shahid.mbc.net/ar/movies/{$name}/movie-player-{$id}", $name, $audio);
    }

    /**
     * @param $url
     * @param null $name
     * @param null $audio
     *
     * @return false
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public static function audioByUrl($url, $name = null, $audio = null)
    {
        if (!$class = self::checkUrlType($url)) {
            console::text("<error>URL is invalid or not supported !</error> : <info>\r\n{$url}</info>");
            return false;
        }
        self::audioBy(new $class($url, $name, null, $audio));
    }

    /**
     * @param shahidable $shahidable
     *
     * @return false
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public static function audioBy(shahidable $shahidable)
    {
        console::text("<info>{$shahidable->getNAME()}</info>");
        console::saveCHCP();
        $PlayLoad = new PlayLoad($shahidable->getID());
        $Manifest = new Manifest($PlayLoad->getM3u8());
        if (!$Manifest->isSuccessful()) {
            console::resetCHCP();
            return false;
        }
        $audio = $Manifest->getAudio($shahidable->getDefaultAudio(), false);
        console::resetCHCP();
        if (!$audio) {
            console::text("<error>No audio file or unselected !</error>");
            return false;
        };
        $fileName = "{$shahidable->getNAME()}.[{$audio->getLANGUAGE()}].aac";
        console::text("<question>{$audio}</question>");
        console::text("<comment>Downloading: </comment><info>{$shahidable->getNAME()} : audio</info>");
        FFMpegCli::run($audio->getURI(), $fileName);
    }

    /**
     * @param int $id
     * @param $name
     * @param null $subtitle
     *
     * @return false
     */
    public static function subtitleById(int $id, $name, $subtitle = null)
    {
        if (1 == 0/*check id*/) {
            console::text("<error>ID is invalid or not supported !</error> : <info>\r\n{$id}</info>");
            return false;
        }
        self::subtitleByUrl("https://shahid.mbc.net/ar/movies/{$name}/movie-player-{$id}", $name, $subtitle);
    }

    /**
     * @param $url
     * @param null $name
     * @param null $subtitle
     *
     * @return false
     */
    public static function subtitleByUrl($url, $name = null, $subtitle = null)
    {
        if (!$class = self::checkUrlType($url)) {
            console::text("<error>URL is invalid or not supported !</error> : <info>\r\n{$url}</info>");
            return false;
        }
        self::subtitleBy(new $class($url, $name, null, null, $subtitle));
    }

    /**
     * @param shahidable $shahidable
     *
     * @return false
     * @throws \GuzzleHttp\Exception\GuzzleException
     */
    public static function subtitleBy(shahidable $shahidable)
    {
        console::text("<info>{$shahidable->getNAME()}</info>");
        console::saveCHCP();
        $PlayLoad = new PlayLoad($shahidable->getID());
        $Manifest = new Manifest($PlayLoad->getM3u8());
        if (!$Manifest->isSuccessful()) {
            console::resetCHCP();
            return false;
        }
        $subtitle = $Manifest->getSubtitle($shahidable->getDefaultSubtitle(), false);
        console::resetCHCP();
        if (!$subtitle) {
            console::text("<error>No subtitle file or unselected !</error>");
            return false;
        };
        $fileName = "{$shahidable->getNAME()}.[{$subtitle->getLANGUAGE()}].srt";
        console::text("<question>{$subtitle}</question>");
        console::text("<comment>Downloading: </comment><info>{$shahidable->getNAME()} : subtitle</info>");
        FFMpegCli::subtitle($subtitle->getURI(), $fileName);
    }
}